import type { Expression, AnyExpression, AccumulatorOperator, AnyObject, ObjectExpressionOperator, WindowOperator } from "./expression";
import type { FilterQuery } from "./query";
/**
 * [Stages reference](https://www.mongodb.com/docs/manual/reference/operator/aggregation-pipeline/#aggregation-pipeline-stages)
 */
export type PipelineStage = PipelineStage.AddFields | PipelineStage.Bucket | PipelineStage.BucketAuto | PipelineStage.CollStats | PipelineStage.Count | PipelineStage.Densify | PipelineStage.Documents | PipelineStage.Facet | PipelineStage.Fill | PipelineStage.GeoNear | PipelineStage.GraphLookup | PipelineStage.Group | PipelineStage.IndexStats | PipelineStage.Limit | PipelineStage.ListSessions | PipelineStage.Lookup | PipelineStage.Match | PipelineStage.Merge | PipelineStage.Out | PipelineStage.PlanCacheStats | PipelineStage.Project | PipelineStage.Redact | PipelineStage.ReplaceRoot | PipelineStage.ReplaceWith | PipelineStage.Sample | PipelineStage.Search | PipelineStage.SearchMeta | PipelineStage.Set | PipelineStage.SetWindowFields | PipelineStage.Skip | PipelineStage.Sort | PipelineStage.SortByCount | PipelineStage.UnionWith | PipelineStage.Unset | PipelineStage.Unwind | PipelineStage.VectorSearch;
export declare namespace PipelineStage {
    interface AddFields {
        /** [`$addFields` reference](https://www.mongodb.com/docs/manual/reference/operator/aggregation/addFields/) */
        $addFields: Record<string, AnyExpression>;
    }
    interface Bucket {
        /** [`$bucket` reference](https://www.mongodb.com/docs/manual/reference/operator/aggregation/bucket/) */
        $bucket: {
            groupBy: Expression;
            boundaries: any[];
            default?: any;
            output?: Record<string, AccumulatorOperator>;
        };
    }
    interface BucketAuto {
        /** [`$bucketAuto` reference](https://www.mongodb.com/docs/manual/reference/operator/aggregation/bucketAuto/) */
        $bucketAuto: {
            groupBy: Expression | Record<string, Expression>;
            buckets: number;
            output?: Record<string, AccumulatorOperator>;
            granularity?: "R5" | "R10" | "R20" | "R40" | "R80" | "1-2-5" | "E6" | "E12" | "E24" | "E48" | "E96" | "E192" | "POWERSOF2";
        };
    }
    interface CollStats {
        /** [`$collStats` reference](https://www.mongodb.com/docs/manual/reference/operator/aggregation/collStats/) */
        $collStats: {
            latencyStats?: {
                histograms?: boolean;
            };
            storageStats?: {
                scale?: number;
            };
            count?: Record<string | number | symbol, never>;
            queryExecStats?: Record<string | number | symbol, never>;
        };
    }
    interface Count {
        /** [`$count` reference](https://www.mongodb.com/docs/manual/reference/operator/aggregation/count/) */
        $count: string;
    }
    interface Densify {
        /** [`$densify` reference](https://www.mongodb.com/docs/manual/reference/operator/aggregation/densify/) */
        $densify: {
            field: string;
            partitionByFields?: string[];
            range: {
                step: number;
                unit?: "millisecond" | "second" | "minute" | "hour" | "day" | "week" | "month" | "quarter" | "year";
                bounds: number[] | globalThis.Date[] | "full" | "partition";
            };
        };
    }
    interface Documents {
        /** [`$documents` reference](https://www.mongodb.com/docs/manual/reference/operator/aggregation/documents/) */
        $documents: Record<string, Expression>[];
    }
    interface Fill {
        /** [`$fill` reference](https://www.mongodb.com/docs/manual/reference/operator/aggregation/fill/) */
        $fill: {
            partitionBy?: Expression;
            partitionByFields?: string[];
            sortBy?: Record<string, 1 | -1>;
            output: Record<string, {
                value: Expression;
            } | {
                method: "linear" | "locf";
            }>;
        };
    }
    interface Facet {
        /** [`$facet` reference](https://www.mongodb.com/docs/manual/reference/operator/aggregation/facet/) */
        $facet: Record<string, FacetPipelineStage[]>;
    }
    type FacetPipelineStage = Exclude<PipelineStage, PipelineStage.CollStats | PipelineStage.Facet | PipelineStage.GeoNear | PipelineStage.IndexStats | PipelineStage.Out | PipelineStage.Merge | PipelineStage.PlanCacheStats>;
    interface GeoNear {
        /** [`$geoNear` reference](https://www.mongodb.com/docs/manual/reference/operator/aggregation/geoNear/) */
        $geoNear: {
            near: {
                type: "Point";
                coordinates: [number, number];
            } | [number, number];
            distanceField: string;
            distanceMultiplier?: number;
            includeLocs?: string;
            key?: string;
            maxDistance?: number;
            minDistance?: number;
            query?: AnyObject;
            spherical?: boolean;
            /**
             * Deprecated. Use only with MondoDB below 4.2 (removed in 4.2)
             * @deprecated
             */
            num?: number;
        };
    }
    interface GraphLookup {
        /** [`$graphLookup` reference](https://www.mongodb.com/docs/manual/reference/operator/aggregation/graphLookup/) */
        $graphLookup: {
            from: string;
            startWith: any;
            connectFromField: string;
            connectToField: string;
            as: string;
            maxDepth?: number;
            depthField?: string;
            restrictSearchWithMatch?: AnyObject;
        };
    }
    interface Group {
        /** [`$group` reference](https://www.mongodb.com/docs/manual/reference/operator/aggregation/group) */
        $group: {
            _id: any;
        } | {
            [key: string]: AccumulatorOperator;
        };
    }
    interface IndexStats {
        /** [`$indexStats` reference](https://www.mongodb.com/docs/manual/reference/operator/aggregation/indexStats/) */
        $indexStats: Record<string | number | symbol, never>;
    }
    interface Limit {
        /** [`$limit` reference](https://www.mongodb.com/docs/manual/reference/operator/aggregation/limit/) */
        $limit: number;
    }
    interface ListSessions {
        /** [`$listSessions` reference](https://www.mongodb.com/docs/manual/reference/operator/aggregation/listSessions/) */
        $listSessions: {
            users?: {
                user: string;
                db: string;
            }[];
        } | {
            allUsers?: true;
        };
    }
    interface Lookup {
        /** [`$lookup` reference](https://www.mongodb.com/docs/manual/reference/operator/aggregation/lookup/) */
        $lookup: {
            from: string;
            as: string;
            localField?: string;
            foreignField?: string;
            let?: Record<string, any>;
            pipeline?: Exclude<PipelineStage, PipelineStage.Merge | PipelineStage.Out>[];
        };
    }
    interface Match {
        /** [`$match` reference](https://www.mongodb.com/docs/manual/reference/operator/aggregation/match/) */
        $match: FilterQuery<any>;
    }
    interface Merge {
        /** [`$merge` reference](https://www.mongodb.com/docs/manual/reference/operator/aggregation/merge/) */
        $merge: {
            into: string | {
                db: string;
                coll: string;
            };
            on?: string | string[];
            let?: Record<string, Expression>;
            whenMatched?: "replace" | "keepExisting" | "merge" | "fail" | Extract<PipelineStage, PipelineStage.AddFields | PipelineStage.Set | PipelineStage.Project | PipelineStage.Unset | PipelineStage.ReplaceRoot | PipelineStage.ReplaceWith>[];
            whenNotMatched?: "insert" | "discard" | "fail";
        };
    }
    interface Out {
        /** [`$out` reference](https://www.mongodb.com/docs/manual/reference/operator/aggregation/out/) */
        $out: string | {
            db: string;
            coll: string;
        };
    }
    interface PlanCacheStats {
        /** [`$planCacheStats` reference](https://www.mongodb.com/docs/manual/reference/operator/aggregation/planCacheStats/) */
        $planCacheStats: Record<string | number | symbol, never>;
    }
    interface Project {
        /** [`$project` reference](https://www.mongodb.com/docs/manual/reference/operator/aggregation/project/) */
        $project: {
            [field: string]: AnyExpression | Expression | Project["$project"];
        };
    }
    interface Redact {
        /** [`$redact` reference](https://www.mongodb.com/docs/manual/reference/operator/aggregation/redact/) */
        $redact: Expression;
    }
    interface ReplaceRoot {
        /** [`$replaceRoot` reference](https://www.mongodb.com/docs/manual/reference/operator/aggregation/replaceRoot/) */
        $replaceRoot: {
            newRoot: AnyExpression;
        };
    }
    interface ReplaceWith {
        /** [`$replaceWith` reference](https://www.mongodb.com/docs/manual/reference/operator/aggregation/replaceWith/) */
        $replaceWith: ObjectExpressionOperator | {
            [field: string]: Expression;
        } | `$${string}`;
    }
    interface Sample {
        /** [`$sample` reference](https://www.mongodb.com/docs/manual/reference/operator/aggregation/sample/) */
        $sample: {
            size: number;
        };
    }
    interface Search {
        /** [`$search` reference](https://www.mongodb.com/docs/atlas/atlas-search/query-syntax/) */
        $search: {
            index?: string;
            highlight?: {
                /** [`highlightPath` reference](https://docs.atlas.mongodb.com/atlas-search/path-construction/#multiple-field-search) */
                path: string | string[] | {
                    value: string;
                    multi: string;
                };
                maxCharsToExamine?: number;
                maxNumPassages?: number;
            };
            [operator: string]: any;
        };
    }
    interface SearchMeta {
        /** [`$searchMeta` reference](https://www.mongodb.com/docs/atlas/atlas-search/query-syntax/#mongodb-pipeline-pipe.-searchMeta) */
        $searchMeta: {
            index?: string;
            highlight?: {
                /** [`highlightPath` reference](https://docs.atlas.mongodb.com/atlas-search/path-construction/#multiple-field-search) */
                path: string | string[] | {
                    value: string;
                    multi: string;
                };
                maxCharsToExamine?: number;
                maxNumPassages?: number;
            };
            [operator: string]: any;
        };
    }
    interface Set {
        /** [`$set` reference](https://www.mongodb.com/docs/manual/reference/operator/aggregation/set/) */
        $set: Record<string, AnyExpression | any>;
    }
    interface SetWindowFields {
        /** [`$setWindowFields` reference](https://www.mongodb.com/docs/manual/reference/operator/aggregation/setWindowFields/) */
        $setWindowFields: {
            partitionBy?: any;
            sortBy?: Record<string, 1 | -1>;
            output: Record<string, WindowOperator & {
                window?: {
                    documents?: [string | number, string | number];
                    range?: [string | number, string | number];
                    unit?: "year" | "quarter" | "month" | "week" | "day" | "hour" | "minute" | "second" | "millisecond";
                };
            }>;
        };
    }
    interface Skip {
        /** [`$skip` reference](https://www.mongodb.com/docs/manual/reference/operator/aggregation/skip/) */
        $skip: number;
    }
    interface Sort {
        /** [`$sort` reference](https://www.mongodb.com/docs/manual/reference/operator/aggregation/sort/) */
        $sort: Record<string, 1 | -1 | Expression.Meta>;
    }
    interface SortByCount {
        /** [`$sortByCount` reference](https://www.mongodb.com/docs/manual/reference/operator/aggregation/sortByCount/) */
        $sortByCount: Expression;
    }
    interface UnionWith {
        /** [`$unionWith` reference](https://www.mongodb.com/docs/manual/reference/operator/aggregation/unionWith/) */
        $unionWith: string | {
            coll: string;
            pipeline?: Exclude<PipelineStage, PipelineStage.Out | PipelineStage.Merge>[];
        } | {
            coll?: string;
            pipeline: Exclude<PipelineStage, PipelineStage.Out | PipelineStage.Merge>[];
        };
    }
    interface Unset {
        /** [`$unset` reference](https://www.mongodb.com/docs/manual/reference/operator/aggregation/unset/) */
        $unset: string | string[];
    }
    interface Unwind {
        /** [`$unwind` reference](https://www.mongodb.com/docs/manual/reference/operator/aggregation/unwind/) */
        $unwind: string | {
            path: string;
            includeArrayIndex?: string;
            preserveNullAndEmptyArrays?: boolean;
        };
    }
    interface VectorSearch {
        /** [`$vectorSearch` reference](https://www.mongodb.com/docs/atlas/atlas-vector-search/vector-search-stage/) */
        $vectorSearch: {
            index: string;
            path: string;
            queryVector: number[];
            numCandidates: number;
            limit: number;
            filter?: Expression;
        };
    }
}
